package com.simuwang.manage.service.impl;

import com.simuwang.base.common.conts.FileSourceType;
import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.common.util.DateUtils;
import com.simuwang.base.mapper.daq.FileManageMapper;
import com.simuwang.base.mapper.daq.FundInfoMapper;
import com.simuwang.base.pojo.dos.FileManageDO;
import com.simuwang.base.pojo.dos.FundInfoDO;
import com.simuwang.base.pojo.dto.query.FileManagePageQuery;
import com.simuwang.base.pojo.vo.FileManageVO;
import com.simuwang.base.pojo.vo.SourceTypeVO;
import com.simuwang.manage.service.FileMangeService;
import com.simuwang.shiro.utils.UserUtils;
import com.smppw.common.pojo.ResultVo;
import com.smppw.common.pojo.enums.status.ResultCode;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.security.SecurityProperties;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.List;
import java.util.stream.Collectors;

/**
 * FileName: FileMangeServiceImpl
 * Author:   chenjianhua
 * Date:     2024/12/19 14:03
 * Description: ${DESCRIPTION}
 */
@Service
public class FileMangeServiceImpl implements FileMangeService {

    private static final Logger logger = LoggerFactory.getLogger(FileMangeServiceImpl.class);

    @Autowired
    private FileManageMapper fileManageMapper;

    @Autowired
    private FundInfoMapper fundInfoMapper;

    @Value("${email.file.path}")
    private String path;
    @Override
    public MybatisPage<FileManageVO> searchFileManage(FileManagePageQuery fileManagePageQuery) {
        List<FileManageDO> fileManageDOList = fileManageMapper.searchFileManage(fileManagePageQuery);
        List<FileManageVO> fileManageVOList = fileManageDOList.stream().map(FileManageDO::toVO).collect(Collectors.toList());
        long total = fileManageMapper.countFileManage(fileManagePageQuery);
        return MybatisPage.of(total,fileManageVOList);
    }

    @Override
    public ResultVo uploadFile(MultipartFile file, String fundId, String fundName, String registerNumber, Integer fileType, String fileDate) {
        ResultVo vo = new ResultVo(true);
        try{
            Integer userId = UserUtils.getLoginUser().getUserId();
            long currentTimeMillis = System.currentTimeMillis();
            String fileName = file.getOriginalFilename();
            //路徑为固定存储目录加用户ID加时间戳
            String filePath = (path.endsWith("/")?path:path+"/")+ userId + "/" +currentTimeMillis+"/";
            File dir = new File(filePath);
            if(!dir.exists()){
                dir.mkdirs();
            }
            File dataFile = new File(filePath+fileName);
            FileUtils.copyToFile(file.getInputStream(),dataFile);
            FileManageDO fileManageDO = new FileManageDO();
            fileManageDO.setFileName(fileName);
            fileManageDO.setFundName(fundName);
            fileManageDO.setFundId(fundId);
            fileManageDO.setRegisterNumber(registerNumber);
            fileManageDO.setFileType(fileType);
            fileManageDO.setFileDate(DateUtils.parse(fileDate,DateUtils.YYYY_MM_DD));
            fileManageDO.setSourceType(FileSourceType.CUSTOM_SOURCE);
            fileManageDO.setFilePath(filePath+fileName);
            //这里使用时间戳加用户ID作为自定义上传文件的ID
            fileManageDO.setFileId(currentTimeMillis+userId);
            fileManageDO.setIsvalid(1);
            fileManageDO.setCreateTime(DateUtils.getNowDate());
            fileManageDO.setUpdateTime(DateUtils.getNowDate());
            fileManageDO.setCreatorId(userId);
            fileManageDO.setUpdaterId(userId);
            fileManageMapper.saveFileManage(fileManageDO);
        }catch (Exception e){
            logger.error(e.getMessage(),e);
            vo.setData(false);
            vo.setCode(ResultCode.SAVE_FAILED.getCode());
        }
        return vo;
    }

    @Override
    public FileManageVO getFileByFileId(Long fileId) {
        FileManageDO fileManageDO = fileManageMapper.getFileInfoByFileId(fileId);
        return fileManageDO.toVO();
    }

    @Override
    public void changeFileSourceType(SourceTypeVO sourceTypeVO) {
        List<Long> fileIdList = sourceTypeVO.getFileIdList();
        Integer fileType = sourceTypeVO.getFileType();
        Integer userId = UserUtils.getLoginUser().getUserId();
        fileManageMapper.changeFileSourceType(fileIdList,fileType,FileSourceType.CUSTOM_SOURCE,userId);
    }

    @Override
    public void changeFileFundMappingInfo(List<Integer> fileIdList, String fundId) {
        try{
            FundInfoDO fundInfoDO = fundInfoMapper.searchFundDetail(fundId);
            Integer userId = UserUtils.getLoginUser().getUserId();
            fileManageMapper.batchUpdateFileManageInfo(fileIdList,fundInfoDO.getFundId(),fundInfoDO.getFundName(),fundInfoDO.getRegisterNumber(),FileSourceType.CUSTOM_SOURCE,userId);
        }catch (Exception e){
            logger.error(e.getMessage(),e);
        }

    }

    @Override
    public void updateFileManage(FileManageVO fileManageVO) {
        Integer userId = UserUtils.getLoginUser().getUserId();
        FileManageDO fileManageDO = fileManageMapper.getFileInfoByFileId(fileManageVO.getFileId());
        FundInfoDO fundInfoDO = fundInfoMapper.searchFundDetail(fileManageVO.getFundId());
        if(fundInfoDO != null){
            fileManageDO.setFundId(fundInfoDO.getFundId());
            fileManageDO.setFundName(fundInfoDO.getFundName());
            fileManageDO.setRegisterNumber(fundInfoDO.getRegisterNumber());
            fileManageDO.setFileType(fileManageVO.getFileType());
        }
        fileManageDO.setUpdaterId(userId);
        fileManageDO.setUpdateTime(DateUtils.getNowDate());
        fileManageMapper.updateFileManage(fileManageDO);
    }
}
